@@ -5,10 +5,18 @@
 #' @return
 #' \code{f <- futureCall(FUN, args)} creates a \link{Future} \code{f} that calls function \code{FUN} with arguments \code{args}, where the value of the future is retrieved using \code{x <- value(f)}.
 #'
+#' @example incl/futureCall.R
+#'
+#' @details
+#' The \code{futureCall()} function works analogously to
+#' \code{\link[base]{do.call}()}, which calls a function with a set of
+#' arguments.  The difference is that \code{do.call()} returns the value of
+#' the call whereas \code{futureCall()} returns a future.
+#'
 #' @rdname future
 #' @importFrom utils capture.output str
 #' @export
-futureCall <- function(FUN, args = NULL, envir = parent.frame(), lazy = FALSE, seed = NULL, globals = TRUE, packages = NULL, evaluator = plan("next"), ...) {
+futureCall <- function(FUN, args = list(), envir = parent.frame(), lazy = FALSE, seed = NULL, globals = TRUE, packages = NULL, evaluator = plan("next"), ...) {
   stop_if_not(is.function(FUN))
   stop_if_not(is.list(args))
 
@@ -63,7 +71,10 @@ futureCall <- function(FUN, args = NULL, envir = parent.frame(), lazy = FALSE, s
   }
   globals <- as.FutureGlobals(globals)
   stop_if_not(inherits(globals, "FutureGlobals"))
-  
+
+  ## Make sure to clean out globals not found
+  globals <- cleanup(globals, drop = "missing")
+
   names <- names(globals)
   if (!is.element("FUN", names)) globals$FUN <- FUN
   if (!is.element("args", names)) globals$args <- args
