@@ -1,7 +1,7 @@
 library("future")
 library("graphics")
 
-plotWhatIsDone <- function(counts) {
+plot_what_is_done <- function(counts) {
   for (kk in seq_along(counts)) {
     f <- counts[[kk]]
 
@@ -10,49 +10,52 @@ plotWhatIsDone <- function(counts) {
 
     ## Not resolved?
     if (!resolved(f)) next
-    
-    cat(sprintf("Plotting tile #%d of %d ...\n", kk, n))
+
+    message(sprintf("Plotting tile #%d of %d ...", kk, n))
     counts[[kk]] <- value(counts[[kk]])
     screen(kk)
     plot(counts[[kk]])
-  } # for (kk ...)
+  }
 
   counts
-} # plotWhatIsDone()
+}
 
 
 ## Options
 region <- getOption("future.demo.mandelbrot.region", 1L)
 if (!is.list(region)) {
   if (region == 1L) {
-    region <- list(xmid=-0.75, ymid=0.0, side=3.0)
+    region <- list(xmid = -0.75, ymid = 0.0, side = 3.0)
   } else if (region == 2L) {
-    region <- list(xmid=0.283, ymid=-0.0095, side=0.00026)
+    region <- list(xmid = 0.283, ymid = -0.0095, side = 0.00026)
   } else if (region == 3L) {
-    region <- list(xmid=0.282989, ymid=-0.01, side=3e-8)
+    region <- list(xmid = 0.282989, ymid = -0.01, side = 3e-8)
   }
 }
 nrow <- getOption("future.demo.mandelbrot.nrow", 3L)
 resolution <- getOption("future.demo.mandelbrot.resolution", 400L)
 delay <- getOption("future.demo.mandelbrot.delay", interactive())
 if (isTRUE(delay)) {
-  delay <- function(counts) Sys.sleep(rexp(1, rate=2))
+  delay <- function(counts) Sys.sleep(rexp(1, rate = 2))
 } else if (!is.function(delay)) {
   delay <- function(counts) {}
 }
 
 ## Generate Mandelbrot tiles to be computed
-Cs <- mandelbrotTiles(xmid=region$xmid, ymid=region$ymid,
-                      side=region$side, nrow=nrow, resolution=resolution)
-
+Cs <- mandelbrot_tiles(xmid = region$xmid, ymid = region$ymid,
+                       side = region$side, nrow = nrow,
+                       resolution = resolution)
 if (interactive()) {
+  if (.Platform$GUI == "RStudio") {
+    if (!"RStudioGD" %in% names(dev.list())) dev.new()
+  }
   dev.new()
   plot.new()
   split.screen(dim(Cs))
   for (ii in seq_along(Cs)) {
     screen(ii)
-    par(mar=c(0,0,0,0))
-    text(x=1/2, y=1/2, sprintf("Future #%d\nunresolved", ii), cex=2)
+    par(mar = c(0, 0, 0, 0))
+    text(x = 1 / 2, y = 1 / 2, sprintf("Future #%d\nunresolved", ii), cex = 2)
   }
 } else {
   split.screen(dim(Cs))
@@ -62,29 +65,29 @@ if (interactive()) {
 counts <- list()
 n <- length(Cs)
 for (ii in seq_len(n)) {
-  cat(sprintf("Mandelbrot tile #%d of %d ...\n", ii, n))
+  message(sprintf("Mandelbrot tile #%d of %d ...", ii, n))
   C <- Cs[[ii]]
-  
+
   counts[[ii]] <- future({
-    cat(sprintf("Calculating tile #%d of %d ...\n", ii, n))
+    message(sprintf("Calculating tile #%d of %d ...", ii, n))
     fit <- mandelbrot(C)
-    
+
     ## Emulate slowness
     delay(fit)
-    
-    cat(sprintf("Calculating tile #%d of %d ... done\n", ii, n))
+
+    message(sprintf("Calculating tile #%d of %d ... done", ii, n))
     fit
   })
 
   ## Plot tiles that are already resolved
-  counts <- plotWhatIsDone(counts)
+  counts <- plot_what_is_done(counts)
 }
 
 
 ## Plot remaining tiles
 repeat {
-  counts <- plotWhatIsDone(counts)
-  if (!any(sapply(counts, FUN=inherits, "Future"))) break
+  counts <- plot_what_is_done(counts)
+  if (!any(sapply(counts, FUN = inherits, "Future"))) break
 }
   
 
